<?php

require_once('PHPUnit/Framework/TestCase.php');


abstract class StandardUnitTestAbstract extends PHPUnit_Framework_TestCase
{
	public $phpParam;
	private $_phpcs = "/usr/bin/phpcs";
	private $_cmdArgs = array(
	    "--standard=src/Maarch",
	    "--report=csv",
	);
	public $sniffs = array();
	
	private function _runPhpcs($filename = ''){
	    $sniffs = (count($this->sniffs) > 0) 
	                    ? ("--sniffs=" . implode(',', $this->sniffs)) 
	                    : '';
		
	    if ($this->phpParam !== null) {
	        $phpcs = 'php -d ' . $this->phpParam . ' ' . $this->_phpcs;
	    } else {
	        $phpcs = $this->_phpcs;
	    }
	    
	    $cmd = $phpcs . ' ' 
		     . implode(' ', $this->_cmdArgs) . ' '
		     . $sniffs . ' '
		     . $filename;
        
		     exec($cmd, $output);
		return $this->_analyseResult($output);
	}
	
	private function _analyseResult($output){
		$csvDelim = ",";
		$csvNumColumns = 7;
		
		$lineColumn = 1;
		$levelColumn = 3;
		$messageColumn = 4;
		$sourceColumn = 5;
		
		array_shift($output);
		
		$result = array(
			"error"   => array(),
			"warning" => array(),
		);
		foreach ($output as &$row){
			$rowData = str_getcsv($row, $csvDelim);
			$tmpLevel = $rowData[$levelColumn];
			$tmpLine = (int) $rowData[$lineColumn];
						
			if (!isset($result[$tmpLevel][$tmpLine])){
				$result[$tmpLevel][$tmpLine] = 1;
			} else {
				$result[$tmpLevel][$tmpLine] += 1;
			}
		}
		
		return $result;
	}
	
	private function _getIncFileForTest(){
		$tmpObj = new ReflectionObject($this);
		$tmpName = str_replace("_", "/", $tmpObj->getName());
		return str_replace('php', 'inc', 'tests/' . $tmpName . ".inc");
	}
	
	public function testSpec(){
		$result = $this->_runPhpcs(
			$this->_getIncFileForTest()
		);
		$expectedErrors = $this->getErrorList();
		$expectedWarnings = $this->getWarningList();
		
		//Errors

		// Pour chaque ligne où des erreurs étaient attendues
		foreach($expectedErrors as $line => $number){
			// Si des erreurs ont été trouvées, 
			if (array_key_exists($line, $result["error"])){
				// on vérifie que c'est le bon nombre
				$this->assertEquals(
					$number, 
					$result["error"][$line], 
					"expected ${number} error(s) at line ${line}, "
					."but " . $result["error"][$line] . " found."
				);
			} else {
				//sinon, on lève une erreur
				$this->assertEquals(
				    $number,
				    0, 
					"expected ${expectedErrors[$line]} error(s) at line ${line}, "
					."but none found."
				);
			}
			unset($result["error"][$line]);
		}
		//pour chaque erreur trouvée, mais non attendue
		foreach ($result["error"] as $line => $number){
			// on lève une erreur
			$this->assertEquals(
		        0,
				$number, 
				"expected no error at line ${line}, "
				."but ${number} found."
			);
		}
		
		
		//Warnings
		
		// Pour chaque ligne où des warnings étaient attendus
		foreach($expectedWarnings as $line => $number){
			// Si des warnings ont été trouvées, 
			if (array_key_exists($line, $result["warning"])){
				// on vérifie que c'est le bon nombre
				$this->assertEquals(
					$number, 
					$result["warning"][$line], 
					"expected ${number} warning(s) at line ${line}, "
					."but " . $result["warning"][$line] . " found."
				);
			} else {
			//si aucun n'a été trouvé, on lève une erreur
				$this->assertEquals(
					$number, 
					0, 
					"expected ${expectedWarnings[$line]} warning(s) at line ${line},"
					."but none found."
				);
			}
			unset($result["warning"][$line]);
		}
		//pour chaque warning trouvée, mais non attendu
		foreach ($result["warning"] as $line => $number){
			// on lève une erreur
			$this->assertEquals(
					$number,
					0, 
					"expected no warning at line ${line}, "
					."but ${number} found."
				);
		}
	}
	
	public abstract function getErrorList();
	public abstract function getWarningList();
}